#include "network-scheduler.h"

void NetworkScheduler::OpenSocketWithRobotSimScheduler() 
  {
    OpenBlockingUDPSocketServerSide(socket, port_base);
  };


void NetworkScheduler::GetSimulationInformation()
  {
    ServerReceiveDataFromSocketBlocking(socket, (char *)&sim_info, msg_size, sizeof(SimulationInformationPacket));

    if( msg_size != sizeof(SimulationInformationPacket) )
      {
	fprintf(stderr, "ERROR READING SIMULATION INFORMATION HEADER (%d bytes instead of %d)\n", msg_size, sizeof(SimulationInformationPacket));
	exit(-1);
      }

    if( sim_info.type != SIMULATION_INFORMATION )
     {
       fprintf(stderr, "WRONG TYPE FOR MSG SIM_INFO: %d\n", sim_info.type);
       exit(-1);
     }

    fprintf(stderr, "Received Simulation info: %d robots, start time %g simulator %s\n", sim_info.robot_num, sim_info.start_time, sim_info.simulator_name);
    node_positions = new Position[sim_info.robot_num];
    current_time   = sim_info.start_time;
  };


void NetworkScheduler::SendNetworkSimInfo(char *sim_name) 
  {
    BuildNetworkSimInformationPacket((char *)&msg_net_info, msg_size, sim_name);

    ServerSendDataToSocket(socket, (char *)&msg_net_info, msg_size);   
  };


void NetworkScheduler::SendNetworkAck(int step)
{
  char msg = (char)step;
  ServerSendDataToSocket(socket, &msg, 1);   
};


void NetworkScheduler::GetNodePositionsUpdate()
  {
    //ServerReceiveFixedDataLenFromSocketBlocking(socket, (char *)&msg_buf, msg_size, sizeof(NodePositionsPacket));
    //memcpy(&msg_positions, msg_buf, sizeof(NodePositionsPacket));

    ServerPeekDataFromSocketBlocking(socket, (char *)&msg_positions, msg_size, sizeof(NodePositionsPacket));

    if( msg_size != sizeof(NodePositionsPacket) )
      {
	fprintf(stderr, "ERROR READING POSITIONS HEADER (%d bytes instead of %d)\n", msg_size, sizeof(NodePositionsPacket));
	exit(-1);
      }

    if( msg_positions.type  != NODE_POSITIONS )
      {
	fprintf(stderr, "WRONG TYPE FOR NODE POSITIONS HEADER: %d\n", msg_positions.type);
	exit(-1);
      }

    /*
    fprintf(stderr, "(%g) Received POS update header (bytes %d [%d]): %d robots\n", msg_positions.timestamp, msg_positions.size, sizeof(Position),
	    msg_positions.robot_num);
    */

    updated_positions = msg_positions.robot_num;
    current_time      = msg_positions.timestamp;

    ServerReceiveDataFromSocketBlocking(socket, (char **)&msg_buf, msg_size, sizeof(NodePositionsPacket) + msg_positions.size);

    //ServerReceiveDataFromSocketBlocking(socket, (char *)&msg_buf, msg_size, msg_positions.size);

    
    if(msg_size != msg_positions.size + sizeof(NodePositionsPacket))
      {
	fprintf(stderr, "ERROR READING POSITIONS (%d bytes instead of %d)\n", msg_size, sizeof(NodePositionsPacket) + msg_positions.size);
	exit(-1);
      }
    else
      {
	//fprintf(stderr, "Position msg: %d bytes\n", msg_size);

	memcpy(node_positions, msg_buf + sizeof(NodePositionsPacket), msg_positions.size);
    
	//ServerReceiveDataFromSocketBlocking(socket, (char *)node_positions, msg_size, msg_positions.size);

	fprintf(stderr, "Received position updates for %d robots \n", updated_positions);
	for(int i=0; i < updated_positions; i++)
	  fprintf(stderr, "[%d] (%g %g %g)\n", node_positions[i].id, node_positions[i].x, node_positions[i].y, node_positions[i].z);
      }
  }

void NetworkScheduler::UpdateNodePositions() 
{
  /* Update the positions of the nodes for whom an update has been received */
};

