

#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/mobility-module.h"
#include "ns3/config-store-module.h"
#include "ns3/wifi-module.h"
#include "ns3/internet-module.h"
#include "ns3/applications-module.h"
#include "ns3/MAPPLE-online-app-helper.h"
#include "ns3/netanim-module.h"
#include <iostream>
#include <fstream>
#include <vector>
#include <string>

// To enable visualizations...
#include "ns3/pyviz.h"

NS_LOG_COMPONENT_DEFINE ("MappleSimpleAdhoc");

using namespace ns3;

void ReceivePacket (Ptr<Socket> socket)
{

	//printf("--------------------------Received one packet!------------------------------\n" );
	//	Address fromAddress;
	//	Ptr<Packet> packet;
	//
	//
	//	packet = socket->RecvFrom(fromAddress);
	//	Ipv4Address address = InetSocketAddress::ConvertFrom (fromAddress).GetIpv4();
	//	NS_LOG_UNCOND("Receive packet from: " << address);

}

class MyApp : public Application
{
public:

	MyApp ();
	virtual ~MyApp();

	void Setup (Ptr<Socket> socket,Ipv4Address destin, uint32_t packetSize, uint32_t nPackets, DataRate dataRate, double uniform);

private:
	virtual void StartApplication (void);
	virtual void StopApplication (void);

	void ScheduleTx (void);
	void SendPacket (void);

	Ptr<Socket>     m_socket;
	Ipv4Address 	  destination;
	uint32_t        m_packetSize;
	uint32_t        m_nPackets;
	DataRate        m_dataRate;
	EventId         m_sendEvent;
	bool            m_running;
	uint32_t        m_packetsSent;
	double myuniform;
	double step;
};

MyApp::MyApp ()
: m_socket (0),
  m_packetSize (0),
  m_nPackets (0),
  m_dataRate (0),
  m_sendEvent (),
  m_running (false),
  m_packetsSent (0),
  myuniform(0.0),
  step(0.01)
{
}

MyApp::~MyApp()
{
	m_socket = 0;
}

void
MyApp::Setup (Ptr<Socket> socket,Ipv4Address dest, uint32_t packetSize, uint32_t nPackets, DataRate dataRate,double interval)
{
	m_socket = socket;
	destination = dest;
	m_packetSize = packetSize;
	m_nPackets = nPackets;
	m_dataRate = dataRate;
	myuniform = interval;
}

void
MyApp::StartApplication (void)
{
	m_running = true;
	m_packetsSent = 0;
	SendPacket ();
}

void
MyApp::StopApplication (void)
{
	m_running = false;

	if (m_sendEvent.IsRunning ())
	{
		Simulator::Cancel (m_sendEvent);
	}

	if (m_socket)
	{
		m_socket->Close ();
	}
}

void
MyApp::SendPacket (void)
{
	if (this->GetNode()->GetId() == 4) {
		std::cout << "[" << Simulator::Now().GetSeconds() << "] send pkt" << std::endl;
	}

	Ptr<Packet> packet = Create<Packet> (m_packetSize);
	m_socket->SendTo (packet,0,InetSocketAddress(destination,9));

	if (++m_packetsSent < m_nPackets)
	{
		ScheduleTx ();
	}
}

void
MyApp::ScheduleTx (void)
{
	if (m_running)
	{
		//for 1024 packet payload this time is good for rate 3 or 2, if you change to 0.3 then there is syncronization problem?
		Time tNext (Seconds(myuniform));//(m_packetSize * 8 / static_cast<double> (m_dataRate.GetBitRate ())));
		//myuniform values is between 0 - 0.5, so each node has their own traffic generation rates..
		m_sendEvent = Simulator::Schedule (tNext, &MyApp::SendPacket, this);
		myuniform += step;
		if ( (myuniform > 1.0) || (myuniform < 0.01) ) {
			step *= (-1);
		}

	}
}

//void CourseChange (std::string context, Ptr<const MobilityModel> model)
//{
//  Vector position = model->GetPosition ();
//  NS_LOG_UNCOND (context << "[" << Simulator::Now().GetSeconds() << "] " <<
//    " x = " << position.x << ", y = " << position.y << ", z = " << position.z);
//}

int main (int argc, char *argv[])
{
	//	LogComponentEnable("Socket",LOG_LEVEL_ALL);
	//		LogComponentEnable ("Socket", LOG_LEVEL_ALL);/
	//		LogComponentEnable ("MAPPLEONLINE", LOG_LEVEL_INFO);
	//	LogComponentEnable("Packet",LOG_LEVEL_INFO);
	//	LogComponentEnable("UdpSocket",LOG_LEVEL_ALL);

	  { PyViz v; }

	UniformVariable uniformvariable(0.0,2.0);
	UniformVariable myappuniform(0.0,1.0);
	UniformVariable myanimation(0,300);
	//std::string phyMode ("DsssRate1Mbps");
	double step = 100;
	double rss = -80;  // -dBm
	uint32_t packetSize = 600; // bytes
	uint32_t numPackets = 6000;
	double interval = 0.3; // seconds
	double simtime= 1000; // 50000; //you can set simulation time here
	uint32_t numNodes = 20; //number of nodes you want //minimum node should be 3..
	bool verbose = false;
	std::string samplesPrefix;
	std::string phyMode ("OfdmRate6Mbps");
	std::string propagationLossModel;


	CommandLine cmd;

	cmd.AddValue ("phyMode", "Wifi Phy mode", phyMode);
	cmd.AddValue ("rss", "received signal strength", rss);
	cmd.AddValue ("packetSize", "size of application packet sent", packetSize);
	cmd.AddValue ("numPackets", "number of packets generated", numPackets);
	cmd.AddValue ("interval", "interval (seconds) between packets", interval);
	cmd.AddValue ("verbose", "turn on all WifiNetDevice log components", verbose);
	cmd.AddValue ("numnodes", "for how many nodes you want to run the script", numNodes);
	cmd.AddValue ("simTime", "Simulation time require to run both script and mapple app", simtime);

	cmd.Parse (argc, argv);
	// Convert to time object
	//	Time interPacketInterval = Seconds (interval);

	// disable fragmentation for frames below 2200 bytes
	//	Config::SetDefault ("ns3::WifiRemoteStationManager::FragmentationThreshold", StringValue ("2200"));
	//	// turn off RTS/CTS for frames below 2200 bytes
	//	Config::SetDefault ("ns3::WifiRemoteStationManager::RtsCtsThreshold", StringValue ("2200"));
	//	// Fix non-unicast data rate to be the same as that of unicast
	//	Config::SetDefault ("ns3::WifiRemoteStationManager::NonUnicastMode",
	//			StringValue (phyMode));
	NetDeviceContainer devices;
	NodeContainer c;
	c.Create (numNodes);

	// The below set of helpers will help us to put together the wifi NICs we want
	WifiHelper wifi;
	if (verbose)
	{
		wifi.EnableLogComponents ();  // Turn on all Wifi logging
	}
	//	wifi.SetStandard (WIFI_PHY_STANDARD_80211b);
	//////////////////////Michal Script Default Settings////////////////////////
	NqosWifiMacHelper wifiMac = NqosWifiMacHelper::Default ();
	wifiMac.SetType ("ns3::AdhocWifiMac");
	YansWifiPhyHelper wifiPhy = YansWifiPhyHelper::Default ();
	YansWifiChannelHelper wifiChannel;
	wifiChannel.SetPropagationDelay ("ns3::ConstantSpeedPropagationDelayModel");
	if (propagationLossModel == "")
		propagationLossModel = "ns3::LogDistancePropagationLossModel";	// default propagation model
	if (propagationLossModel == "ns3::TwoRayGroundPropagationLossModel") {
		wifiChannel.AddPropagationLoss ("ns3::TwoRayGroundPropagationLossModel",		// TwoRayGroundPropagationLossModel
				"HeightAboveZ",DoubleValue (0.225));
	} else if (propagationLossModel == "ns3::FriisPropagationLossModel") {
		wifiChannel.AddPropagationLoss ("ns3::FriisPropagationLossModel",				// FriisPropagationLossModel
				"SystemLoss",DoubleValue(115.0));
	} else if (propagationLossModel == "ns3::NakagamiPropagationLossModel") {			// LogDistancePropagationLossModel + NakagamiPropagationLossModel
		wifiChannel.AddPropagationLoss ("ns3::LogDistancePropagationLossModel");
		wifiChannel.AddPropagationLoss ("ns3::NakagamiPropagationLossModel",
				"m0",DoubleValue(5.0),
				"m1",DoubleValue(5.0),
				"m2",DoubleValue(5.0));
	} else {
		wifiChannel.AddPropagationLoss (propagationLossModel);
	}
	wifiPhy.SetChannel (wifiChannel.Create ());
	wifi = WifiHelper::Default ();
	wifi.SetRemoteStationManager ("ns3::ConstantRateWifiManager", "DataMode", StringValue ("OfdmRate6Mbps"), "RtsCtsThreshold", UintegerValue (0));
	devices = wifi.Install (wifiPhy, wifiMac, c);




	MobilityHelper mobility;
	mobility.SetPositionAllocator ("ns3::GridPositionAllocator",
			"MinX", DoubleValue (50.0), //50x,50y with 30 step and 100 pos.x,y //now 40 old 50 work// now 80 old 40 work
			"MinY", DoubleValue (50.0), //now increase the pos.set
			"DeltaX", DoubleValue (step),
			"DeltaY", DoubleValue (step),
			"GridWidth", UintegerValue (5),
			"LayoutType", StringValue ("RowFirst"));
	mobility.SetMobilityModel ("ns3::ConstantPositionMobilityModel");

//	// Using RandomWaypoint mobility model
//	ObjectFactory pos;
//	pos.SetTypeId ("ns3::RandomRectanglePositionAllocator");
//	pos.Set ("X", RandomVariableValue (UniformVariable (0.0, 200))); //100 works // 150 minx is 40// old 100 now 150 with 20 step and minx 80 not work
//	pos.Set ("Y", RandomVariableValue (UniformVariable (0.0, 200))); //sample collected at 400 metter //but small node nomber need less distance
//	Ptr<PositionAllocator> ptrPositionAlloc = pos.Create ()->GetObject<PositionAllocator> ();
//	mobility.SetMobilityModel ("ns3::RandomWaypointMobilityModel",
//			"Speed",RandomVariableValue(ConstantVariable(1.0)),
//			"Pause",RandomVariableValue(ConstantVariable(10.0)),
//			"PositionAllocator",PointerValue(ptrPositionAlloc));

	//what actually means of speed and pause see later, but for 10.0 10.0 its not working , NeighborStateTimer condition make true,
	//only works for small speeds like 0.01 or 0.05 etc. Ask Michal?
	//	mobility.SetPositionAllocator ("ns3::RandomRectanglePositionAllocator",
	//			"X", RandomVariableValue(UniformVariable(0,200)),
	//			"Y", RandomVariableValue(UniformVariable(0,200)));
	//	mobility.SetMobilityModel ("ns3::WaypointMobilityModel",
	//			"InitialPositionIsWaypoint",BooleanValue(true));


	mobility.Install (c);

	InternetStackHelper internet;
	internet.Install (c);

	Ipv4AddressHelper ipv4;
	NS_LOG_INFO ("Assign IP Addresses.");
	ipv4.SetBase ("10.1.1.0", "255.255.255.0");
	Ipv4InterfaceContainer interfaces = ipv4.Assign (devices);

	//Here I need to send traffic and receive traffic on same node

	TypeId tid = TypeId::LookupByName ("ns3::UdpSocketFactory");
	Ptr<Socket> recvSink[numNodes];
	//	InetSocketAddress local;

	for (uint32_t counter=0;counter<numNodes;counter++){

		recvSink[counter] = Socket::CreateSocket (c.Get (counter), tid);
		InetSocketAddress local = InetSocketAddress (Ipv4Address::GetAny(), 9);
		recvSink[counter]->Bind (local);
		recvSink[counter]->SetRecvCallback(MakeCallback(&ReceivePacket));
	}

	MappleOnlineApplicationkHelper appHelper1("ns3::UdpSocketFactory", 8080 ,interfaces,c,samplesPrefix,simtime);
	Ipv4Address destination;
	Ptr<MyApp> app[numNodes];
	Ptr<Socket> ns3UdpSocket[numNodes];
	ApplicationContainer app1[numNodes];

	for(uint32_t j=0;j<numNodes;j=j+3){
		double interval = uniformvariable.GetValue();
		printf("Value of intervale for j:%u is:%f",j,interval);
		ns3UdpSocket[j] = Socket::CreateSocket (c.Get (j), tid);
		destination = Ipv4Address::GetBroadcast();
		ns3UdpSocket[j]->SetAllowBroadcast(true);
		app[j] = CreateObject<MyApp> ();
		// destination = interfaces.GetAddress(GetNode()->GetId()).GetBroadcast();
		//here 1024 byte is packet length..and send 350000 number of packets// 0.2 second is interval to send the packet
		app[j]->Setup (ns3UdpSocket[j],destination, 100,3500000, DataRate ("1Mbps"),0.05);
		c.Get (j)->AddApplication (app[j]);

		app1[j]= appHelper1.Install(c.Get(j));

		app[j]->SetStartTime (Seconds (interval));
		app1[j].Start(Seconds (myappuniform.GetValue()));
		app[j]->SetStopTime (Seconds (simtime));
		app1[j].Stop(Seconds(simtime));
	}
	for(uint32_t j=1;j<numNodes;j=j+3){
			double interval = uniformvariable.GetValue();
	//		printf("Value of intervale for j:%u is:%f",j,interval);
			ns3UdpSocket[j] = Socket::CreateSocket (c.Get (j), tid);
			destination = Ipv4Address::GetBroadcast();
			ns3UdpSocket[j]->SetAllowBroadcast(true);
			app[j] = CreateObject<MyApp> ();
			// destination = interfaces.GetAddress(GetNode()->GetId()).GetBroadcast();
			//here 100 byte is packet length..and send 350000 number of packets// 0.2 second is interval to send the packet
			app[j]->Setup (ns3UdpSocket[j],destination, 100, 3500000, DataRate ("1Mbps"),0.05);
			c.Get (j)->AddApplication (app[j]);

			app1[j]= appHelper1.Install(c.Get(j));

			app[j]->SetStartTime (Seconds (interval));
			app1[j].Start(Seconds (myappuniform.GetValue()));
			app[j]->SetStopTime (Seconds (simtime));
			app1[j].Stop(Seconds(simtime));
		}

	for(uint32_t j=2;j<numNodes;j=j+3){
				double interval = uniformvariable.GetValue();
		//		printf("Value of intervale for j:%u is:%f",j,interval);
				ns3UdpSocket[j] = Socket::CreateSocket (c.Get (j), tid);
				destination = Ipv4Address::GetBroadcast();
				ns3UdpSocket[j]->SetAllowBroadcast(true);
				app[j] = CreateObject<MyApp> ();
				// destination = interfaces.GetAddress(GetNode()->GetId()).GetBroadcast();
				//here 600 byte is packet length..and send 250000 number of packets// 0.3 second is interval to send the packet
				app[j]->Setup (ns3UdpSocket[j],destination, 100, 2500000, DataRate ("1Mbps"),0.05);
				c.Get (j)->AddApplication (app[j]);

				app1[j]= appHelper1.Install(c.Get(j));

				app[j]->SetStartTime (Seconds (interval));
				app1[j].Start(Seconds (myappuniform.GetValue()));
				app[j]->SetStopTime (Seconds (simtime));
				app1[j].Stop(Seconds(simtime));
			}
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	//
	//	for(uint32_t index=0;index<numNodes;index++){
	//		app1[index]= appHelper1.Install(c.Get(index));
	//
	//
	//		app1[index].Start(Seconds (uniformvariable.GetValue()));
	//		//app1[index].Stop(Seconds (100.0));
	//
	//	}

	//Ptr<Node> n = c.Get (1);

	//anim.SetMobilityPollInterval(Seconds(1));
	// anim.SetRandomPosition(true);
	// Show nodes initial positions
	//	  Ptr<Node> node = c.Get (0);
	//	  Ptr<MobilityModel> mob = node->GetObject<MobilityModel> ();
	//	  for(uint32_t i = 0; i < 20; i++) {
	//		  node = c.Get (i);
	//		  //anim.s (node, myanimation.GetValue(), myanimation.GetValue());
	//		  mob = node->GetObject<MobilityModel> ();
	//		  std::ostringstream oss;
	//		  oss << "Node" << i << " initial position:\t";
	//		  Simulator::Schedule (Seconds (0.001), &CourseChange, oss.str() , mob);
	//	  }
//	AnimationInterface anim;
	//if(Simulator::Now().GetSeconds() > 500 ){
//		AnimationInterface	anim("Mapplsimpleadhoc.xml");
//	}
//	if(Simulator::Now().GetSeconds() > 500 ){
//		anim.StopAnimation();
//	}
	//	  anim.SetMobilityPollInterval(Seconds(1));

	Simulator::Run ();
	Simulator::Destroy ();
	//anim.StopAnimation();

	return 0;
}

