

#ifndef ANTORDLRNEIGHBOR_H
#define ANTORDLRNEIGHBOR_H

#include "ns3/simulator.h"
#include "ns3/timer.h"
#include "ns3/ipv4-address.h"
#include "ns3/callback.h"
#include "ns3/wifi-mac-header.h"
#include "ns3/arp-cache.h"
#include <vector>

namespace ns3
{
namespace antordlr
{
class AntORDLRCore;
/**
 * \ingroup antor-dnr
 * \brief maintain list of active neighbors
 */
class Neighbors
{
public:
  /// c-tor
  Neighbors(Time delay);
  /// Neighbor description
  struct Neighbor
  {
      Ipv4Address m_neighborAddress;
      Mac48Address m_hardwareAddress;
      Time m_expireTime;
      bool close;

      Neighbor (Ipv4Address ip, Mac48Address mac, Time t) :
        m_neighborAddress (ip), m_hardwareAddress (mac), m_expireTime (t),
        close (false)
      {

      }

  };
  /// Return expire time for neighbor node with address addr, if exists, else return 0.
  Time GetExpireTime (Ipv4Address addr);
  /// Check that node with address addr  is neighbor
  bool IsNeighbor (Ipv4Address addr);
  /// Update expire time for entry with address addr, if it exists, else add new entry
  void Update (Ipv4Address addr, Time expire);
  /// Remove all expired entries
  void Purge ();
  /// Schedule m_ntimer.
  void ScheduleTimer ();
  /// Remove all entries
  void Clear () { m_nb.clear (); }
  

  void EraseNeighbor (uint32_t index) {m_nb.erase(m_nb.begin() + index);}
  bool FindNeighbor (Ipv4Address addr, uint32_t &idx);

  std::vector<Ipv4Address> GetNeighbors() {
	  std::vector<Ipv4Address> _neigh;
	  for (uint32_t b=0; b < m_nb.size() ; b++){
		  _neigh.push_back(m_nb[b].m_neighborAddress);
	  }

	 return _neigh;

  }

  /// Add ARP cache to be used to allow layer 2 notifications processing
  void AddArpCache (Ptr<ArpCache>);

  /// Don't use given ARP cache any more (interface is down)
  void DelArpCache (Ptr<ArpCache>);

   ///\name Handle link failure callback
  //\{
  /// Get callback to ProcessTxError
   Callback<void, WifiMacHeader const &> GetTxErrorCallback () const { return m_txErrorCallback; }
  void SetCallback (Callback<void, Ipv4Address> cb) { m_handleLinkFailure = cb;}
  Callback<void, Ipv4Address> GetCallback () const { return m_handleLinkFailure; }
  //\}

private:
  /// link failure callback
  Callback<void, Ipv4Address> m_handleLinkFailure;

  /// Timer for neighbor's list. Schedule Purge().
  Timer m_ntimer;
  /// vector of entries
  std::vector<Neighbor> m_nb;

  /// TX error callback
   Callback<void, WifiMacHeader const &> m_txErrorCallback;

  /// list of ARP cached to be used for layer 2 notifications processing
   std::vector<Ptr<ArpCache> > m_arp;

   /// Find MAC address by IP using list of ARP caches
   Mac48Address LookupMacAddress (Ipv4Address);
   /// Process layer 2 TX error notification
   void ProcessTxError (WifiMacHeader const &);
};

}
}

#endif /* ANTORDNRNEIGHBOR_H */
