/*
 * Copyright 2012 IDSIA
 *
 *
 * Author:  Michal Kudelski
 *
 * (based on packet-sink-helper.h and .cc)
 *
 */

#include "MAPPLE-app-helper.h"
#include "ns3/MAPPLE-app.h"
#include "ns3/string.h"
#include "ns3/inet-socket-address.h"
#include "ns3/names.h"
#include "ns3/uinteger.h"

namespace ns3 {

MappleApplicationkHelper::MappleApplicationkHelper (std::string protocol, uint16_t portBase, Ipv4InterfaceContainer inter, NodeContainer nc, std::string prefix, std::string model)
{
  m_factory.SetTypeId ("ns3::MappleApplication");
  m_factory.Set ("Protocol", StringValue (protocol));
  m_factory.Set("port", UintegerValue(portBase));

  this -> iface = inter;
  this -> nodes = nc;
  this -> prefix = prefix;
  this -> model = model;
}

void
MappleApplicationkHelper::SetAttribute (std::string name, const AttributeValue &value)
{
  m_factory.Set (name, value);
}

ApplicationContainer
MappleApplicationkHelper::Install (Ptr<Node> node) const
{
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
MappleApplicationkHelper::Install (std::string nodeName) const
{
  Ptr<Node> node = Names::Find<Node> (nodeName);
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
MappleApplicationkHelper::Install (NodeContainer c) const
{
  ApplicationContainer apps;
  for (NodeContainer::Iterator i = c.Begin (); i != c.End (); ++i)
    {
      apps.Add (InstallPriv (*i));
    }

  return apps;
}

Ptr<Application>
MappleApplicationkHelper::InstallPriv (Ptr<Node> node) const
{
  Ptr<Application> app = m_factory.Create<Application> ();
  Ptr<MappleApplication> mappleApp = DynamicCast<MappleApplication>(app);
  mappleApp->SetInterfaceContainer(this->iface);
  mappleApp->SetNodeContainer(this->nodes);
  if ( (this->prefix) != "" )
	  mappleApp->SetPrefix(this->prefix);
  if ( (this->model) != "" )
	  mappleApp->SetModel(this->model);
  node->AddApplication (app);

  return app;
}

} // namespace ns3
