 #include "ns3-network-application.h"

#include "ns3/ptr.h"
#include "ns3/data-rate.h"
#include "ns3/nstime.h"
#include "ns3/node.h"
#include "ns3/socket.h"
#include "ns3/inet-socket-address.h"
#include "ns3/application.h"
#include "ns3/socket-factory.h"
#include "ns3/simulator.h"
#include "ns3/packet.h"
#include "ns3/random-variable.h"
//#include "ns3/object.h"
#include "ns3/udp-socket-factory.h"
#include "ns3/udp-socket.h"
#include "ns3/uinteger.h"
#include "ns3/double.h"
#include "ns3/address-utils.h"
#include "ns3/log.h"

#include "swarmanoidDefines.h"
#include "networkApplicationTag.h"
#include "ns3/trace-source-accessor.h"
#include "ns3/random-variable.h"


using namespace ns3;
using namespace std;
namespace IDSIA{

  uint16_t NetworkApplication::staticId = 0;
  NS_OBJECT_ENSURE_REGISTERED (NetworkApplication);
  NS_LOG_COMPONENT_DEFINE("NetworkApplication");
  TypeId NetworkApplication::GetTypeId(void){
    static TypeId tid = TypeId("IDSIA::NetworkApplication")
      .SetParent<Application>()
      .AddConstructor<NetworkApplication>()
      .AddAttribute("portBase", "Port number on which the schedulers are connected",
		    UintegerValue(1025),
		    MakeUintegerAccessor(&NetworkApplication::portBase),
		    MakeUintegerChecker<uint16_t>(1))
      .AddAttribute("numberOfRobots", "The number of robots present in the simulation",
		    UintegerValue(10),
		    MakeUintegerAccessor(&NetworkApplication::numberOfRobots),
		    MakeUintegerChecker<uint16_t>(1))
      .AddAttribute("port", "Port number on which the ns3 <-> ns3 socket comunicates",
		    UintegerValue(1025),
		    MakeUintegerAccessor(&NetworkApplication::port),
		    MakeUintegerChecker<uint16_t>(1))
      .AddAttribute("simulationStep", "The simulation step used to synchronize Argos and ns3",
		    DoubleValue(0.1),
		    MakeDoubleAccessor(&NetworkApplication::simulationStep),
		    MakeDoubleChecker<double>(0.1))
      .AddTraceSource ("Tx", "A new packet is created and is sent",
		       MakeTraceSourceAccessor (&NetworkApplication::m_txTrace))
      .AddTraceSource ("Rx", "A new packet is received",
		       MakeTraceSourceAccessor (&NetworkApplication::m_rxTrace));

    return tid;
  }



  NetworkApplication::NetworkApplication(){
    //std::cerr << "StaticId = " << staticId << std::endl;
    appId = staticId++;
    //std::cerr << "appId = " << appId << std::endl;
    max_content_size = MAX_MSG_SIZE;


    //setting the 2 randomvariable generators

    m_onTime = ConstantVariable(1.0);
    m_offTime = ConstantVariable(0.0);
    //this->InitSocketWithRobotApplication(); //Non funziona, Binding: Permission Denied, porta nn settata
  }
  NetworkApplication::~NetworkApplication(){}

  void NetworkApplication::StartApplication(void){
    sendSocket = ns3::Socket::CreateSocket(GetNode(),TypeId::LookupByName("ns3::UdpSocketFactory"));
    sendSocket->Bind();
    sendSocket->SetRecvCallback(MakeCallback(&IDSIA::NetworkApplication::Receive,this));
    sendSocket->SetSendCallback(MakeCallback(&IDSIA::NetworkApplication::Sent,this));
    bool sendBroadcast =  sendSocket->GetAllowBroadcast();
    if (!sendBroadcast)
    	sendSocket->SetAllowBroadcast (true);

    sinkSocket = ns3::Socket::CreateSocket(GetNode(), TypeId::LookupByName("ns3::UdpSocketFactory"));
    Ipv4Address dstaddr = interfaces.GetAddress(this->appId);
    InetSocketAddress dst = InetSocketAddress(dstaddr,port);
    sinkSocket->Bind(dst);
    sinkSocket->SetRecvCallback(MakeCallback(&IDSIA::NetworkApplication::Receive,this));
    sinkSocket->SetSendCallback(MakeCallback(&IDSIA::NetworkApplication::Sent,this));
    bool receiveBroadcast = sinkSocket->GetAllowBroadcast();
    if (!receiveBroadcast)
    	sinkSocket->SetAllowBroadcast (true);

    //calling the rng
    m_onTime.GetValue();
    m_offTime.GetValue();

  }

  void NetworkApplication::StopApplication(void){
    if(sendSocket != NULL){
      //sinkSocket->SetRecvCallback(MakeCallback(NULL));//setting the recv callback to null
      sendSocket->SetSendCallback((Callback<void, Ptr<ns3::Socket>, uint32_t>) NULL); //setting the send callback to null
    }
  }

  void NetworkApplication::DoDispose(void){
    //close the ns3 socket
    if(sinkSocket != NULL){
      sinkSocket->Close();
    }
    if(sendSocket != NULL){
      sendSocket->Close();
    }
    //sending the call up
    ns3::Application::DoDispose();
  }


  void NetworkApplication::Receive(Ptr<ns3::Socket> socket){
#if PRINT
    std::cerr << Simulator::Now().GetSeconds() << " Node " << appId << " received \n" ;
#endif
    //estrapolazione del contenuto del pacchetto
    Address from;
    Ptr<Packet> packet = sinkSocket->RecvFrom(from);
    uint8_t buf[packet->GetSize()];
    packet->CopyData(buf,packet->GetSize());
    m_rxTrace (packet);
    //std::cerr << "\tReceived Msg is: " << buf << std::endl;

    InetSocketAddress address = InetSocketAddress::ConvertFrom(from);
    int32_t src  = -1;
    for(uint32_t i = 0; i<interfaces.GetN();i++){
      if(interfaces.GetAddress(i) == address.GetIpv4()){
	src = i;
	break;
      }
    }
    //std::cerr << "From Node: " << src << std::endl;
    //printf("[%f] %d received msg from %d\n",Simulator::Now().GetSeconds(),sinkSocket->GetNode()->GetId(),src);
    std::string str_content((const char*)buf,packet->GetSize());
    SendReceivedDataToRobotApp(Simulator::Now().GetSeconds(),src, str_content);
  }


  void NetworkApplication::Sent(Ptr<ns3::Socket> socket,  uint32_t length){ //callback x invio (serve x debug)
    //std::cerr << Simulator::Now().GetSeconds() << " Socket sended " << length << " bytes\n";
  }


  void NetworkApplication::ScheduleNextMsgReading(void){
    //this->ReadMsgFromArgos();
  }

  void NetworkApplication::InitSocketWithRobotApplication(){
    //this->max_content_size = _max_content_size;

    //apertura del socket verso il robot
    #if DEBUG
    std::cerr << "Node: " << this->appId << " / " << GetNode() << " / " << this
	      << " opening connection to port: " << portBase+(appId+1)
	      << " portBase: " << portBase << std::endl;
    #endif
    OpenBlockingUDPSocketServerSide(robot2ns3Socket,portBase+(appId+1));
    this->msg_content = new char[this->max_content_size];
    this->received_data = new char[max_content_size];
  }


  void NetworkApplication::GetStartInfoFromRobotAppl(){
    #if DEBUG
    std::cerr << "Time: " << Simulator::Now().GetSeconds() << " waiting on robot2ns3 socket, node: " << appId << std::endl;
    #endif
    ServerReceiveDataFromSocketBlocking(robot2ns3Socket,(char *)&msg_start, msg_size, sizeof(RobotStartPacket));
    //std::cerr << "Node: " << appId << " received back\n";
    //Controllo errori
    if(msg_size != sizeof(RobotStartPacket)){
      fprintf(stderr, "WRONG SIZE FROM ROBOT START PACKET (size %d instead of %lu\n", msg_size, sizeof(RobotStartPacket));
      exit(-1);
    }
    if(msg_start.type != START_FROM_ROBOT_PROCESS){
      fprintf(stderr,"WRONG START PACKET (type %d)\n", msg_start.type);
      exit(-1);
    }
    else{
#if PRINT
      fprintf(stderr, "[%d] Received START from robot %d (%s)\n", appId, msg_start.id, msg_start.robot_type);
#endif
    }
    //scheduling the first search for new msgs
    Simulator::Schedule(Seconds(0.0), &NetworkApplication::AcquireAndSendMessages, this);
  }



  vector<list<string> > NetworkApplication::GetNewInputDataFromRobotApp(Real current_time_slice, int num_robots){
    char num_msgs;
    vector<list<string> > vecRecv(num_robots);
    ServerReceiveDataFromSocketBlocking(robot2ns3Socket, &num_msgs, msg_size, 1);
    #if DEBUG
    fprintf(stderr, "Node %d receiving %d messages from robot simulator\n", appId, num_msgs);
    #endif
    for(int i=0;i<num_msgs;i++){
      ReceiveData(current_time_slice);
      if(msg_header.to == -1){ /*-1  means a broadcast message*/
	msg_header.to = this->appId;/*A message to yourself means broadcast*/
      }
      string str_content(msg_content + sizeof(ApplicationPacket),msg_size-sizeof(ApplicationPacket));
      vecRecv[msg_header.to].push_back(str_content);
    }
    return vecRecv;
  }


  void NetworkApplication::ReceiveData(Real current_time_slice){
    ServerReceiveDataFromSocketBlocking(robot2ns3Socket, &msg_content, msg_size, MAX_MSG_SIZE + sizeof(ApplicationPacket));
    memcpy(&msg_header, msg_content, sizeof(ApplicationPacket));
    #if DEBUG
    fprintf(stderr, "[%d @ %g] Received an APPLICATION Msg sent from robot %d at %g to robot %d (size %d bytes)\n",
	    appId, current_time_slice, msg_header.from, msg_header.timestamp, msg_header.to, msg_header.size);
    fprintf(stderr, "\t Msg content: [%s]\n", msg_content+sizeof(ApplicationPacket));
    #endif
  }

  void NetworkApplication::SendReceivedDataToRobotApp(Real current_time_slice, int from, string content){
    GenerateMsg(&received_data, msg_size, from, appId, content, current_time_slice);
    ServerSendDataToSocket(robot2ns3Socket, received_data, msg_size);
  }

  void NetworkApplication::AcquireAndSendMessages(void){
	  std::vector<list<std::string> > vecMsgs;
	  Real current_time_slice = (float)Simulator::Now().GetSeconds();
	  vecMsgs = this->GetNewInputDataFromRobotApp(current_time_slice, numberOfRobots);
	  //Iterating over the messages
	  for(uint32_t i=0;i<numberOfRobots;i++){
		  if(!vecMsgs.at(i).empty()){
#if DEBUG
			  std::cerr << Simulator::Now().GetSeconds() << " Node " << appId << " has new msg to send to " << i << std::endl;
#endif
			  for(list<string>::iterator k=vecMsgs.at(i).begin();k!=vecMsgs.at(i).end();k++){
				  this->CreateAndSendPacket(*k,i,appId);
			  }

		  }
	  }
	  //scheduling the next acquiring and sending operation
	  Simulator::Schedule(Seconds(this->simulationStep),&IDSIA::NetworkApplication::AcquireAndSendMessages, this);
}

  void NetworkApplication::CreateAndSendPacket(std::string content, int destinationId, unsigned long int delay){
#if DEBUG
    std::cerr << "Time (s): " << Simulator::Now().GetSeconds() << " Node " << this->appId << " ready to Send\n";
#endif
    //creating the packet to send over the network
    Ptr<Packet> packet = Create<Packet> (reinterpret_cast<const uint8_t*>(content.c_str()),content.size()+1);
    //uint32_t sizeGap = PACKETSIZE - packet->GetSize();
    //packet->AddPaddingAtEnd(sizeGap);			// TODO FIXME to add here or not - make a decision !!!!!!
    //std::cerr << "packet->GetSize(): " << packet->GetSize() << std::endl;
    NetworkApplicationTag nat;
    nat.SetSender(this->appId);
    packet->AddPacketTag(nat);
    //Getting the address of the destination
    Ipv4Address destination;
    if(destinationId == appId){
      destination = interfaces.GetAddress(destinationId).GetBroadcast();
    }else{
      destination = interfaces.GetAddress(destinationId);
    }
#if DEBUG
    std::cerr << "Destination Address is: " << destination;
#endif
    //sending (Michal added: with delay, as many robots sending at _exactly_ the same time may cause unexpected interferences...)
    Simulator::Schedule (MicroSeconds(delay),&NetworkApplication::DelayedSendSocket, this, packet,destination,port);
    //int bytes = sendSocket -> SendTo(packet, 0, InetSocketAddress(destination,port));
    //printf("[%f]%d sent a packet to %d\n",Simulator::Now().GetSeconds(),appId,destinationId);
#if DEBUG
    std::cerr << " Sent " << bytes << " bytes\n";
#endif
    m_txTrace (packet);

  }

  void NetworkApplication::DelayedSendSocket(Ptr<Packet> packet, Ipv4Address destination,uint16_t port) {
	  sendSocket -> SendTo(packet, 0, InetSocketAddress(destination,port));
  }

}



