#ifndef PFRROUTINGPROTOCOL_H
#define PFRROUTINGPROTOCOL_H

#include "ns3/node.h"
#include "ns3/uinteger.h"
#include "ns3/double.h"
#include "pfr-dpd.h"
#include "ns3/ipv4-routing-protocol.h"
#include "ns3/ipv4-interface.h"
#include "ns3/ipv4-l3-protocol.h"


namespace ns3
{
namespace pfr
{

class RoutingProtocol : public Ipv4RoutingProtocol
{
public:
	  static TypeId GetTypeId (void);
	  //static const uint32_t PFR_PORT;

	  /// c-tor
	  RoutingProtocol ();
	  virtual ~RoutingProtocol();
	  virtual void DoDispose ();

	  ///\name From Ipv4RoutingProtocol
	  //\{
	  Ptr<Ipv4Route> RouteOutput (Ptr<Packet> p, const Ipv4Header &header, Ptr<NetDevice> oif, Socket::SocketErrno &sockerr);
	  bool RouteInput (Ptr<const Packet> p, const Ipv4Header &header, Ptr<const NetDevice> idev,
	                   UnicastForwardCallback ucb, MulticastForwardCallback mcb,
	                   LocalDeliverCallback lcb, ErrorCallback ecb);
	  virtual void NotifyInterfaceUp (uint32_t interface);
	  virtual void NotifyInterfaceDown (uint32_t interface);
	  virtual void NotifyAddAddress (uint32_t interface, Ipv4InterfaceAddress address);
	  virtual void NotifyRemoveAddress (uint32_t interface, Ipv4InterfaceAddress address);
	  virtual void SetIpv4 (Ptr<Ipv4> ipv4);
	  virtual void PrintRoutingTable (Ptr<OutputStreamWrapper> stream) const;
	  //\}

	  ///\name Handle protocol parameters
	  //\{
	  uint32_t GetParameter1 () const { return Parameter1; }
	  void SetParameter1 (uint32_t p);
	  double GetBroadcastProbability () const { return BroadcastProbability; }
	  void SetBroadcastProbability (double bp);
	  void SetBroadcastEnable (bool f) { EnableBroadcast = f; }
	  bool GetBroadcastEnable () const { return EnableBroadcast; }
	  //\}

	  void DelayedUcbCallback(UnicastForwardCallback ucb,Ptr<Ipv4Route> route, Ptr<const Packet> p, const Ipv4Header &header);

private:
  ///\name Protocol parameters.
  //\{
  uint32_t Parameter1;             ///< Parameter no. 1

  double BroadcastProbability;             ///< Probability that the node will broadcast a packet (when performing probabilistic flooding)

  bool EnableBroadcast;              ///< Indicates whether a a broadcast data packets forwarding enable

  double IdCacheMemoryTime;					///< How long packet IDs are going to be remembered to avoid routing loops during flooding...
  //\}

  /// IP protocol
  Ptr<Ipv4> m_ipv4;
  /// Loopback device used to defer RREQ until packet will be fully formed
  Ptr<NetDevice> m_lo;

  /// Handle duplicated broadcast/multicast packets
   DuplicatePacketDetection m_dpd;

};




}  // namespace pfr
}  // namespace ns3
#endif /* PFRROUTINGPROTOCOL_H */
